%----------------------------------------------%
%   Parser problem			       %
%   Standard Prolog version		       %
%   with windows			       %
%   written by Peter Kacsuk		       %
%----------------------------------------------%

%   calling parameters: N number of word lists (max. 20)
%			D delay time

problem(N,D):-
   is_int(N), N > 0, N < 21,
   is_int(D), D > 0,
   start(N,D).
problem(_,_):-
   write("Sorry, parameter error.").

start(N,Delay):-
   set_value(delay_value,Delay),
   create_arcs,
   create_deactivated_windows,
   create_activated_windows,
   create_result_window,
   draw_arcs,
   draw_parse_tree,
   cpu_time(ST),
   M is N + 1,
   do(1,M,ST).

do(M,M,ST):-
   cpu_time(ET),
   T is ET - ST,
   display_time(T).
do(N,M,ST):-
   find_clause(word_list(N,L)),
   sentence(L,R),
   display_result(L,R,N),
   MM is N + 1 ,
   do(MM,M,ST).

sentence(X,succ):-
   activate_window(s1a,"sentence",X),
   get_value(delay_value,D),
   delay(D),
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   noun_phrase(Y,n2a,n2,"noun_ph."),
   verb_phrase(Z,v3a,v3,"verb_ph."),
   deactivate_window(s1,"sentence"),
   !.
sentence(X,fail):-
   deactivate_window(s1,"sentence").

noun_phrase(X,Wa,Wd,Name):-
   activate_window(Wa,Name,X),
   get_value(delay_value,D),
   delay(D),
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   next_left(Wd,Wla,Wld,Nl),
   determiner_check(Y,Wla,Wld,Nl),
   next_right(Wd,Wra,Wrd,Nr),
   noun_check(Z,Wra,Wrd,Nr),
   deactivate_window(Wd,Name), !.
noun_phrase(X,Wa,Wd,Name):-
   deactivate_window(Wd,Name), fail.

verb_phrase(X,Wa,Wd,Name):-
   activate_window(Wa,Name,X),
   get_value(delay_value,D),
   delay(D),
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   next_left(Wd,Wla,Wld,Nl),
   verb_check(Y,Wla,Wld,Nl),
   next_right(Wd,Wra,Wrd,Nr),
   noun_phrase(Z,Wra,Wrd,Nr),
   deactivate_window(Wd,Name), !.
verb_phrase(X,Wa,Wd,Name):-
   activate_window(Wa,Name,X),
   verb_check(X,v8a,v8,"verb"),
   deactivate_window(Wd,Name), !.
verb_phrase(X,Wa,Wd,Name):-
   deactivate_window(Wd,Name), fail.

determiner_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X),
   get_value(delay_value,D),
   delay(D),
   determiner(X),
   deactivate_window(Wd,N), !.
determiner_check(_,_,Wd,N):-
   deactivate_window(Wd,N), fail.

determiner([the]).

noun_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X),
   get_value(delay_value,D),
   delay(D),
   noun(X),
   deactivate_window(Wd,N), !.
noun_check(_,_,Wd,N):-
   deactivate_window(Wd,N), fail.

noun([man]).
noun([pork]).
noun([wine]).

verb_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X),
   get_value(delay_value,D),
   delay(D),
   verb(X),
   deactivate_window(Wd,N), !.
verb_check(_,_,Wd,N):-
   deactivate_window(Wd,N), fail.

verb([eats]).
verb([drinks]).

append([],L,L).
append([X|L1],L2,[X|L3]):-
	append(L1,L2,L3).

delay(0):- !.
delay(N):-
   M is N - 1 ,
   delay(M).

word_list(1,[the,man,eats,pork]).
word_list(2,[the,good,man]).
word_list(3,[the,man,eats]).
word_list(4,[man,drinks,wine]).
word_list(5,[the,man,eats,the,pork]).
word_list(6,[man,eats,pork]).
word_list(7,[man,eats,man]).
word_list(8,[the,man,eats,the,pork]).
word_list(9,[the,man,eats,the,boiled,pork]).
word_list(10,[man,drinks,pork]).
word_list(11,[man,eats,the,pork]).
word_list(12,[the,man,eats,the,beef]).
word_list(13,[the,pork,the,beef]).
word_list(14,[the,man,wine]).
word_list(15,[the,man,drinks,the,wine]).
word_list(16,[the,pork,drinks,the,dry,wine]).
word_list(17,[the,man,and,pork,drinks,wine]).
word_list(18,[man,drinks,the,wine]).
word_list(19,[man,drinks,beer]).
word_list(20,[the,man,and,the,pork,eats,wine]).

%----------------------------------------------%
%	window handler			       %
%----------------------------------------------%

draw_parse_tree:-
   deactivate_window(s1,"sentence"),
   deactivate_window(n2,"noun_ph."),
   deactivate_window(v3,"verb_ph."),
   deactivate_window(d4,"determiner"),
   deactivate_window(n5,"noun"),
   deactivate_window(v6,"verb"),
   deactivate_window(n7,"noun_ph."),
   deactivate_window(v8,"verb"),
   deactivate_window(d9,"determiner"),
   deactivate_window(n10,"noun").

create_deactivated_windows:-
   create_window(2,20,3,40,628,s1),
   create_window(7,5,3,30,628,n2),
   create_window(7,45,3,30,628,v3),
   create_window(13,3,3,12,628,d4),
   create_window(13,19,3,12,628,n5),
   create_window(13,35,3,12,628,v6),
   create_window(13,51,3,12,628,n7),
   create_window(13,67,3,12,628,v8),
   create_window(18,43,3,12,628,d9),
   create_window(18,59,3,12,628,n10).

create_activated_windows:-
   create_window(2,20,3,40,544,s1a),
   create_window(7,5,3,30,544,n2a),
   create_window(7,45,3,30,544,v3a),
   create_window(13,3,3,12,544,d4a),
   create_window(13,19,3,12,544,n5a),
   create_window(13,35,3,12,544,v6a),
   create_window(13,51,3,12,544,n7a),
   create_window(13,67,3,12,544,v8a),
   create_window(18,43,3,12,544,d9a),
   create_window(18,59,3,12,544,n10a).

create_result_window:-
   create_window(22,5,3,60,590,res),
   open_window(res),
   write_window(res,0,12,"RESULT").

deactivate_window(Wid,Name):-
   open_window(Wid),
   write_window(Wid,0,1,Name).

activate_window(Wid,Name,L):-
   write_to_string(SL,L),
   open_window(Wid),
   write_window(Wid,0,1,Name),
   write_window(Wid,1,1,SL).

display_result(L,R,N):-
   open_window(res),
   write_to_string(SR,R),
   write_window(res,1,1,SR),
   write_window(res,1,5,":"),
   write_to_string(SN,N),
   write_window(res,1,7,SN),
   write_to_string(SL,L),
   write_window(res,1,10,SL),
   sound(500,5).

next_left(n2,d4a,d4,"determiner").
next_left(v3,v6a,v6,"verb").
next_left(n7,d9a,d9,"determiner").

next_right(n2,n5a,n5,"noun").
next_right(v3,n7a,n7,"noun_ph.").
next_right(n7,n10a,n10,"noun").

create_arcs:-
   create_window(4,25,4,30,275,w1),
   create_window(7,25,3,30,21,w2),
   create_window(9,10,5,19,275,w3),
   create_window(13,10,3,19,21,w4),
   create_window(8,42,6,15,275,w5),
   create_window(13,42,3,15,21,w6),
   create_window(8,56,6,22,275,w7),
   create_window(13,56,3,22,21,w8),
   create_window(15,53,4,8,275,w9),
   create_window(18,53,3,8,21,w10).

draw_arcs:-
   open_window(w1),
   open_window(w2),
   open_window(w3),
   open_window(w4),
   open_window(w5),
   open_window(w6),
   open_window(w7),
   open_window(w8),
   open_window(w9),
   open_window(w10).

display_time(T):-
   create_window(2,1,3,15,590,wt),
   open_window(wt),
   write_window(wt,0,7,"time"),
   write_to_string(ST,T),
   write_window(wt,1,1,ST).

