%----------------------------------------------%
%   Parser problem			       %
%   Parallel Prolog version		       %
%   Improved Pipeline Solution		       %
%   with graphics			       %
%   written by Peter Kacsuk		       %
%----------------------------------------------%

%   calling parameters: N numbers of word_lists
%			D delay value

problem(N,D):-
   is_int(N), N > 0, N < 21,
   is_int(D), D > 0,
   start(N,D).
problem(_,_):-
   write("Sorry, parameter error.").

start(N,D):-
   create_deactivated_windows,
   create_activated_windows,
   create_result_window,
   set_value(delay_value,1),
   draw_deactivated_windows,
   solve_problem(N,D).

solve_problem(N,D):-
   cpu_time(ST),
   M is N + 1,
   new(noun_phrase_process(D,1,0),noun_phrase(1),_,_,1),
   new(noun_phrase_process(D,2,0),noun_phrase(2),_,_,2),
   new(verb_phrase_process(D,0),verb_phrase,_,_,3),
   new(sentence(1,M,D,ST),sentence,_,_,4).

%---------------------------------%
%	sentence process	  %
%---------------------------------%

sentence(N,MAX,D,ST):-
   N < MAX, !,
   set_value(delay_value,D),
   find_clause(word_list(N,L)),
   activate_window(aw4,"sentence",L),
   delete_partition(list_pairs),
   generate_list_pairs(L,M),
   send_list_pairs(1),
   set_value(result,fail),
   collect_answers(M),
   get_value(result,R),
   display_result(L,R,N),
   deactivate_window2(aw4,"sentence"),
   MM is N + 1,
   sentence(MM,MAX,D,ST).
sentence(_,_,_,ST):-
   cpu_time(ET),
   T is ET - ST,
   display_time(T),
   terminate_system.

generate_list_pairs(X,_):-
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   add_clause(list_pairs(Y,Z)),
   fail.
generate_list_pairs(_,N):-
   clause_count(list_pairs,N).

send_list_pairs(N):-
   find_clause(list_pairs(Y,Z),I), !,
   delete_clause(list_pairs,I),
   send(do(Y,Z),[noun_phrase(N)]),
   M is (N mod 2) + 1,
   send_list_pairs(M).
send_list_pairs(_).

collect_answers(0):- !.
collect_answers(N):-
   wait_for_unb(X),
   check_message(X),
   M is N - 1,
   collect_answers(M).

check_message(succ):-
   set_value(result,succ).
check_message(_).

word_list(1,[the,man,eats,pork]).
word_list(2,[the,good,man]).
word_list(3,[the,man,eats]).
word_list(4,[man,drinks,wine]).
word_list(5,[the,man,eats,the,pork]).
word_list(6,[man,eats,pork]).
word_list(7,[man,eats,man]).
word_list(8,[the,man,eats,the,pork]).
word_list(9,[the,man,eats,the,boiled,pork]).
word_list(10,[man,drinks,pork]).
word_list(11,[man,eats,the,pork]).
word_list(12,[the,man,eats,the,beef]).
word_list(13,[the,pork,the,beef]).
word_list(14,[the,man,wine]).
word_list(15,[the,man,drinks,the,wine]).
word_list(16,[the,pork,drinks,the,dry,wine]).
word_list(17,[the,man,and,pork,drinks,wine]).
word_list(18,[man,drinks,the,wine]).
word_list(19,[man,drinks,beer]).
word_list(20,[the,man,and,the,pork,eats,wine]).

%---------------------------------%
%	noun_phrase process	  %
%---------------------------------%

noun_phrase_process(D,T,Cin):-
   set_value(delay_value,D),
   wait_for_unb(do(L,P)),
   write_to_string(ST,T),
   concat("aw",ST,AWid),
   concat("w",ST,Wid),
   activate_window(AWid,"noun_phrase",L),
   noun_phrase(L,R,Cin,Cout),
   write_to_string(SC,Cout),
   deactivate_window(Wid,"noun_phrase",SC),
   send(do(R,P),[verb_phrase]),
   noun_phrase_process(D,T,Cout).	       % endless process

noun_phrase(X,succ,Cin,Cout):-
   C is Cin + 1,
   set_value(counter,C),
   get_value(delay_value,D),
   delay(D),
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, determiner(Y),
   delay_counter, noun(Z),
   get_value(counter,Cout),
   !.
noun_phrase(X,fail,_,Cout):-
   get_value(counter,Cout).

determiner([the]).

noun([man]).
noun([pork]).
noun([wine]).

%---------------------------------%
%	verb_phrase process	  %
%---------------------------------%

verb_phrase_process(D,Cin):-
   set_value(delay_value,D),
   wait_for_unb(do(R,L)),
   action(R,L,Cin,Cout),
   verb_phrase_process(D,Cout).

action(fail,_,Cin,Cin):- !,
   send(fail,[sentence]).
action(succ,L,Cin,Cout):-
   activate_window(aw3,"verb_phrase",L),
   verb_phrase(L,R,Cin,Cout),
   write_to_string(SC,Cout),
   deactivate_window(w3,"verb_phrase",SC),
   send(R,[sentence]).

verb_phrase(X,succ,Cin,Cout):-
   C is Cin + 1,
   set_value(counter,C),
   get_value(delay_value,D),
   delay(D),
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, verb(Y),
   noun_phrase(Z),
   get_value(counter,Cout),
   !.
verb_phrase(X,succ,_,Cout):-
   delay_counter, verb(X),
   get_value(counter,Cout),
   !.
verb_phrase(X,fail,_,Cout):-
   get_value(counter,Cout).

verb([eats]).
verb([drinks]).

noun_phrase(X):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, determiner(Y),
   delay_counter, noun(Z).

%---------------------------------%
%	common data base	  %
%---------------------------------%

append([],L,L).
append([X|L1],L2,[X|L3]):-
	append(L1,L2,L3).

delay_counter:-
   get_value(counter,Cin),
   C is Cin + 1,
   set_value(counter,C),
   get_value(delay_value,D),
   delay(D).

delay(0):- !.
delay(N):-
   M is N - 1 ,
   delay(M).

%----------------------------------------------%
%		Window handler		       %
%----------------------------------------------%

init_windows:-
   create_deactivated_windows,
   create_activated_windows,
   draw_deactivated_windows.

create_deactivated_windows:-
   create_window(3,35,17,40,275,w11),
   create_window(7,20,9,30,275,w12),
   create_window(5,5,3,60,628,w4),
   create_window(10,3,3,30,628,w1),
   create_window(10,37,3,30,628,w2),
   create_window(15,5,3,60,628,w3).

create_activated_windows:-
   create_window(5,5,3,60,560,aw4),
   create_window(10,3,3,30,608,aw1),
   create_window(10,37,3,30,592,aw2),
   create_window(15,5,3,60,544,aw3).

draw_deactivated_windows:-
   deactivate_window2(w11,""),
   deactivate_window2(w12,""),
   deactivate_window(w4,"sentence","0"),
   deactivate_window(w1,"noun_phrase","0"),
   deactivate_window(w2,"noun_phrase","0"),
   deactivate_window(w3,"verb_phrase","0").

deactivate_window(Wid,Name,Count):-
   deactivate_window2(Wid,Name),
   write_window(Wid,0,2,Count).

deactivate_window2(Wid,Name):-
   open_window(Wid),
   write_window(Wid,0,11,Name).

activate_window(Wid,Name,L):-
   write_to_string(SL,L),
   open_window(Wid),
   write_window(Wid,0,11,Name),
   write_window(Wid,1,1,SL).

create_result_window:-
   create_window(22,5,3,60,590,res),
   open_window(res),
   write_window(res,0,12,"RESULT").

display_result(L,R,N):-
   open_window(res),
   write_to_string(SR,R),
   write_window(res,1,1,SR),
   write_window(res,1,5,":"),
   write_to_string(SN,N),
   write_window(res,1,7,SN),
   write_to_string(SL,L),
   write_window(res,1,10,SL),
   sound(500,5).

display_time(T):-
   create_window(1,1,3,15,590,wt),
   open_window(wt),
   write_window(wt,0,7,"time"),
   write_to_string(ST,T),
   write_window(wt,1,1,ST).

