%----------------------------------------------%
%   Parser problem			       %
%   Parallel Prolog version		       %
%   Pipeline Solution			       %
%   with graphics			       %
%   written by Peter Kacsuk		       %
%----------------------------------------------%

%   calling parameters: N numbers of word_lists
%			D delay value

problem(N,D):-
   is_int(N), N > 0, N < 21,
   is_int(D), D > 0,
   start(N,D).
problem(_,_):-
   write("Sorry, parameter error.").

start(N,D):-
   create_deactivated_windows,
   create_activated_windows,
   create_result_window,
   draw_deactivated_windows,
   new(main(N,D),main).

main(N,D):-
   cpu_time(ST),
   delete_partition(n_counter),
   delete_partition(v_counter),
   add_clause(n_counter(0)),
   add_clause(v_counter(0)),
   problem(1,N,D,ST).

problem(N,M,D,ST):-
   new(sentence(N),sentence,_,_,4),
   n_counter(NC),
   new(noun_phrase_process(D,NC),noun_phrase,_,_,2),
   v_counter(VC),
   new(verb_phrase_process(D,VC),verb_phrase,_,_,3),
   delete_clause(n_counter,1),
   delete_clause(v_counter,1),
   wait_for_unb(end),
   wait_for_unb(nc(NC2)),
   add_clause(n_counter(NC2)),
   wait_for_unb(vc(VC2)),
   add_clause(v_counter(VC2)),
   fail.
problem(M,M,_,ST):-
   cpu_time(ET),
   T is ET - ST,
   display_time(T).
problem(N,M,D,ST):-
   MM is N + 1,
   problem(MM,M,D,ST).

%---------------------------------%
%	sentence process	  %
%---------------------------------%

sentence(N):-
   find_clause(word_list(N,L)),
   activate_window(aw2,"sentence",L),
   delete_partition(list_pairs),
   generate_list_pairs(L,M),
   send_list_pairs,
   set_value(result,fail),
   collect_answers(M),
   get_value(result,R),
   display_result(L,R,N),
   delay(D),
   deactivate_window2(w2,"sentence"),
   send_unb(end,[main,noun_phrase,verb_phrase]).

generate_list_pairs(X,_):-
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   add_clause(list_pairs(Y,Z)),
   fail.
generate_list_pairs(_,N):-
   clause_count(list_pairs,N).

send_list_pairs:-
   find_clause(list_pairs(Y,Z),I), !,
   delete_clause(list_pairs,I),
   send_unb(do(Y,Z),[noun_phrase]),
   send_list_pairs.
send_list_pairs.

collect_answers(0):- !.
collect_answers(N):-
   wait_for_unb(X),
   check_message(X),
   M is N - 1,
   collect_answers(M).

check_message(succ):- !,
   set_value(result,succ).
check_message(_).

word_list(1,[the,man,eats,pork]).
word_list(2,[the,good,man]).
word_list(3,[the,man,eats]).
word_list(4,[man,drinks,wine]).
word_list(5,[the,man,eats,the,pork]).
word_list(6,[man,eats,pork]).
word_list(7,[man,eats,man]).
word_list(8,[the,man,eats,the,pork]).
word_list(9,[the,man,eats,the,boiled,pork]).
word_list(10,[man,drinks,pork]).
word_list(11,[man,eats,the,pork]).
word_list(12,[the,man,eats,the,beef]).
word_list(13,[the,pork,the,beef]).
word_list(14,[the,man,wine]).
word_list(15,[the,man,drinks,the,wine]).
word_list(16,[the,pork,drinks,the,dry,wine]).
word_list(17,[the,man,and,pork,drinks,wine]).
word_list(18,[man,drinks,the,wine]).
word_list(19,[man,drinks,beer]).
word_list(20,[the,man,and,the,pork,eats,wine]).

%---------------------------------%
%	noun_phrase process	  %
%---------------------------------%

noun_phrase_process(D,NC):-
   set_value(delay_value,D),
   set_value(counter,NC),
   wait_for_unb(X),
   noun_do(X,D).

noun_do(do(L,P),D):- !,
   activate_window(aw3,"noun_phrase",L),
   noun_phrase(L,R),
   get_value(counter,Cout),
   write_to_string(SC,Cout),
   deactivate_window(w3,"noun_phrase",SC),
   send_unb(do(R,P),[verb_phrase]),
   noun_phrase_process(D,Cout).      % repeat
noun_do(end,_):-		     % finish
   get_value(counter,NC),
   send_unb(nc(NC),[main]).

noun_phrase(X,succ):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, determiner(Y),
   delay_counter, noun(Z),
   !.
noun_phrase(X,fail).

determiner([the]).

noun([man]).
noun([pork]).
noun([wine]).

%---------------------------------%
%	verb_phrase process	  %
%---------------------------------%

verb_phrase_process(D,VC):-
   set_value(delay_value,D),
   set_value(counter,VC),
   wait_for_unb(X),
   verb_do(X,D).

verb_do(do(R,L),D):-
   action(R,L),
   get_value(counter,VC),
   verb_phrase_process(D,VC).
verb_do(end,_):-
   get_value(counter,VC),
   send_unb(vc(VC),[main]).

action(fail,_):- !,
   send_unb(fail,[sentence]).
action(succ,L):-
   activate_window(aw4,"verb_phrase",L),
   verb_phrase(L,R),
   get_value(counter,Cout),
   write_to_string(SC,Cout),
   deactivate_window(w4,"verb_phrase",SC),
   send_unb(R,[sentence]).

verb_phrase(X,succ):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, verb(Y),
   noun_phrase(Z),
   !.
verb_phrase(X,succ):-
   delay_counter, verb(X),
   !.
verb_phrase(X,fail).

verb([eats]).
verb([drinks]).

noun_phrase(X):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   delay_counter, determiner(Y),
   delay_counter, noun(Z).

%---------------------------------%
%	common data base	  %
%---------------------------------%

append([],L,L).
append([X|L1],L2,[X|L3]):-
	append(L1,L2,L3).

delay_counter:-
   get_value(counter,Cin),
   C is Cin + 1,
   set_value(counter,C),
   get_value(delay_value,D),
   delay(D).

delay(0):- !.
delay(N):-
   M is N - 1 ,
   delay(M).

%----------------------------------------------%
%		Window handler		       %
%----------------------------------------------%

init_windows:-
   create_deactivated_windows,
   create_activated_windows,
   draw_deactivated_windows.

create_deactivated_windows:-
   create_window(3,35,17,40,275,w1),
   create_window(5,5,3,60,628,w2),
   create_window(10,5,3,60,628,w3),
   create_window(15,5,3,60,628,w4).

create_activated_windows:-
   create_window(5,5,3,60,560,aw2),
   create_window(10,5,3,60,592,aw3),
   create_window(15,5,3,60,544,aw4).

draw_deactivated_windows:-
   deactivate_window2(w1,""),
   deactivate_window(w2,"sentence","0"),
   deactivate_window(w3,"noun_phrase","0"),
   deactivate_window(w4,"verb_phrase","0").

deactivate_window(Wid,Name,Count):-
   deactivate_window2(Wid,Name),
   write_window(Wid,0,2,Count).

deactivate_window2(Wid,Name):-
   open_window(Wid),
   write_window(Wid,0,11,Name).

activate_window(Wid,Name,L):-
   write_to_string(SL,L),
   open_window(Wid),
   write_window(Wid,0,11,Name),
   write_window(Wid,1,1,SL).

create_result_window:-
   create_window(22,5,3,60,590,res),
   open_window(res),
   write_window(res,0,12,"RESULT").

display_result(L,R,N):-
   open_window(res),
   write_to_string(SR,R),
   write_window(res,1,1,SR),
   write_window(res,1,5,":"),
   write_to_string(SN,N),
   write_window(res,1,7,SN),
   write_to_string(SL,L),
   write_window(res,1,10,SL),
   sound(500,5).

display_time(T):-
   create_window(1,1,3,15,590,wt),
   open_window(wt),
   write_window(wt,0,7,"time"),
   write_to_string(ST,T),
   write_window(wt,1,1,ST).

