
%----------------------------------------------%
%   Parser problem			       %
%   Parallel Prolog version		       %
%   with static process structure	       %
%	(sentence,noun_phrase,verb_phrase)     %
%   coarse-grain AND-parallelism	       %
%   with windows			       %
%   written by Peter Kacsuk		       %
%----------------------------------------------%

%   calling parameters: N numbers of word_lists
%			D delay value

problem(N,D):-
   is_int(N), N > 0, N < 21,
   is_int(D), D > 0,
   start(N,D).
problem(_,_):-
   write("Sorry, parameter error.").

start(N,D):-
   create_arcs,
   create_deactivated_windows,
   create_activated_windows,
   create_result_window,
   draw_arcs,
   draw_parse_tree,
   solve_problem(N,D).

solve_problem(N,D):-
   cpu_time(ST),
   M is N + 1,
   new(noun_phrase_process(D,0),noun_phrase,_,_,2),
   new(verb_phrase_process(D,0),verb_phrase,_,_,3),
   new(sentence(1,M,D,ST),sentence,_,_,1).

%---------------------------------%
%	sentence process	  %
%---------------------------------%

sentence(N,MAX,D,ST):-
   N < MAX, !,
   set_value(delay_value,D),
   find_clause(word_list(N,L)),
   activate_window(s1a,"sentence",L,11),
   check_sentence(L,R),
   display_result(L,R,N),
   delay(D),
   deactivate_window2(s1,"sentence",11),
   M is N + 1,
   sentence(M,MAX,D,ST).
sentence(_,_,_,ST):-
   cpu_time(ET),
   T is ET - ST,
   display_time(T),
   terminate_system.

check_sentence(X,succ):-
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   send_unb(do(Y),[noun_phrase]),
   send_unb(do(Z),[verb_phrase]),
   wait_for_unb(res(R1)),
   wait_for_unb(res(R2)),
   succ_answer(R1),
   succ_answer(R2).
check_sentence(_,fail).

succ_answer(succ).

word_list(1,[the,man,eats,pork]).
word_list(2,[the,good,man]).
word_list(3,[the,man,eats]).
word_list(4,[man,drinks,wine]).
word_list(5,[the,man,eats,the,pork]).
word_list(6,[man,eats,pork]).
word_list(7,[man,eats,man]).
word_list(8,[the,man,eats,the,pork]).
word_list(9,[the,man,eats,the,boiled,pork]).
word_list(10,[man,drinks,pork]).
word_list(11,[man,eats,the,pork]).
word_list(12,[the,man,eats,the,beef]).
word_list(13,[the,pork,the,beef]).
word_list(14,[the,man,wine]).
word_list(15,[the,man,drinks,the,wine]).
word_list(16,[the,pork,drinks,the,dry,wine]).
word_list(17,[the,man,and,pork,drinks,wine]).
word_list(18,[man,drinks,the,wine]).
word_list(19,[man,drinks,beer]).
word_list(20,[the,man,and,the,pork,eats,wine]).

%---------------------------------%
%	noun_phrase process	  %
%---------------------------------%

noun_phrase_process(D,Cin):-
   set_value(delay_value,D),
   set_value(counter,Cin),
   wait_for_unb(do(L)),
   activate_window(n2a,"noun_ph.",L,11),
   noun_phrase(L,R,Cout),
   write_to_string(SC,Cout),
   deactivate_window(n2,"noun_ph.",SC,11),
   send_unb(res(R),[sentence]),
   noun_phrase_process(D,Cout). 	     % endless process

noun_phrase(X,succ,Cout):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   determiner_check(Y,d4a,d4,"determiner"),
   noun_check(Z,n5a,n5,"noun"),
   get_value(counter,Cout),
   !.
noun_phrase(X,fail,Cout):-
   get_value(counter,Cout).

determiner_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X,1),
   delay_counter,
   determiner(X),
   deactivate_window2(Wd,N,1), !.
determiner_check(_,_,Wd,N):-
   deactivate_window2(Wd,N,1), fail.

determiner([the]).

noun_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X,1),
   delay_counter,
   noun(X),
   deactivate_window2(Wd,N,1), !.
noun_check(_,_,Wd,N):-
   deactivate_window2(Wd,N,1), fail.

noun([man]).
noun([pork]).
noun([wine]).

%---------------------------------%
%	verb_phrase process	  %
%---------------------------------%

verb_phrase_process(D,Cin):-
   set_value(delay_value,D),
   set_value(counter,Cin),
   wait_for_unb(do(L)),
   activate_window(v3a,"verb_ph.",L,11),
   verb_phrase(L,R,Cout),
   write_to_string(SC,Cout),
   deactivate_window(v3,"verb_ph.",SC,11),
   send_unb(res(R),[sentence]),
   verb_phrase_process(D,Cout).

verb_phrase(X,succ,Cout):-
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   verb_check(Y,v6a,v6,"verb"),
   noun_phrase(Z),
   get_value(counter,Cout),
   !.
verb_phrase(X,succ,Cout):-
   verb_check(X,v8a,v8,"verb"),
   get_value(counter,Cout),
   !.
verb_phrase(X,fail,Cout):-
   get_value(counter,Cout).

verb_check(X,Wa,Wd,N):-
   activate_window(Wa,N,X,1),
   delay_counter,
   verb(X),
   deactivate_window2(Wd,N,1), !.
verb_check(_,_,Wd,N):-
   deactivate_window2(Wd,N,1), fail.

verb([eats]).
verb([drinks]).

noun_phrase(X):-
   activate_window(n7a,"noun_ph.",X,1),
   delay_counter,
   append(Y,Z,X), not(eq(Y,[])), not(eq(Z,[])),
   determiner_check(Y,d9a,d9,"determiner"),
   noun_check(Z,n10a,n10,"noun"),
   deactivate_window2(n7,"noun_ph.",1),!.
noun_phrase(X):-
   deactivate_window2(n7,"noun_ph.",1),
   fail.

%---------------------------------%
%	common data base	  %
%---------------------------------%

append([],L,L).
append([X|L1],L2,[X|L3]):-
	append(L1,L2,L3).

delay_counter:-
   get_value(counter,Cin),
   C is Cin + 1,
   set_value(counter,C),
   get_value(delay_value,D),
   delay(D).

delay(0):- !.
delay(N):-
   M is N - 1 ,
   delay(M).

%----------------------------------------------%
%	window handler			       %
%----------------------------------------------%

draw_parse_tree:-
   deactivate_window2(s1,"sentence",11),
   deactivate_window2(n2,"noun_ph.",11),
   deactivate_window2(v3,"verb_ph.",11),
   deactivate_window2(d4,"determiner",1),
   deactivate_window2(n5,"noun",1),
   deactivate_window2(v6,"verb",1),
   deactivate_window2(n7,"noun_ph.",1),
   deactivate_window2(v8,"verb",1),
   deactivate_window2(d9,"determiner",1),
   deactivate_window2(n10,"noun",1).

create_deactivated_windows:-
   create_window(2,20,3,40,628,s1),
   create_window(7,5,3,30,628,n2),
   create_window(7,45,3,30,628,v3),
   create_window(13,3,3,12,628,d4),
   create_window(13,19,3,12,628,n5),
   create_window(13,35,3,12,628,v6),
   create_window(13,51,3,12,628,n7),
   create_window(13,67,3,12,628,v8),
   create_window(18,43,3,12,628,d9),
   create_window(18,59,3,12,628,n10).

create_activated_windows:-
   create_window(2,20,3,40,560,s1a),
   create_window(7,5,3,30,544,n2a),
   create_window(7,45,3,30,592,v3a),
   create_window(13,3,3,12,544,d4a),
   create_window(13,19,3,12,544,n5a),
   create_window(13,35,3,12,592,v6a),
   create_window(13,51,3,12,592,n7a),
   create_window(13,67,3,12,592,v8a),
   create_window(18,43,3,12,592,d9a),
   create_window(18,59,3,12,592,n10a).

deactivate_window(Wid,Name,Count,Col):-
   deactivate_window2(Wid,Name,Col),
   write_window(Wid,0,2,Count).

deactivate_window2(Wid,Name,Col):-
   open_window(Wid),
   write_window(Wid,0,Col,Name).

activate_window(Wid,Name,L,Col):-
   write_to_string(SL,L),
   open_window(Wid),
   write_window(Wid,0,Col,Name),
   write_window(Wid,1,1,SL).

create_result_window:-
   create_window(22,5,3,60,590,res),
   open_window(res),
   write_window(res,0,12,"RESULT").

display_result(L,R,N):-
   open_window(res),
   write_to_string(SR,R),
   write_window(res,1,1,SR),
   write_window(res,1,5,":"),
   write_to_string(SN,N),
   write_window(res,1,7,SN),
   write_to_string(SL,L),
   write_window(res,1,10,SL),
   sound(500,5).

create_arcs:-
   create_window(4,25,4,30,275,w1),
   create_window(7,25,3,30,21,w2),
   create_window(9,10,5,19,275,w3),
   create_window(13,10,3,19,21,w4),
   create_window(8,42,6,15,275,w5),
   create_window(13,42,3,15,21,w6),
   create_window(8,56,6,22,275,w7),
   create_window(13,56,3,22,21,w8),
   create_window(15,53,4,8,275,w9),
   create_window(18,53,3,8,21,w10).

draw_arcs:-
   open_window(w1),
   open_window(w2),
   open_window(w3),
   open_window(w4),
   open_window(w5),
   open_window(w6),
   open_window(w7),
   open_window(w8),
   open_window(w9),
   open_window(w10).

display_time(T):-
   create_window(2,1,3,15,590,wt),
   open_window(wt),
   write_window(wt,0,7,"time"),
   write_to_string(ST,T),
   write_window(wt,1,1,ST).

