%---------------------------------------%
%    Maze problem			%
%    written by 			%
%	Peter Kacsuk and Agnes Zubora	%
%---------------------------------------%

problem:-
   draw( 1 ),
   draw( 2 ),
   draw( 3 ),
   draw( 4 ),
   get_way_list( WL),
   start( S ),
   del_pos( S, WL, WL2 ),
   step( S, WL2, [] ).

get_way_list( [ 6:3, 5:3, 4:3, 3:5, 4:5, 5:5, 6:2,
	       6:5, 3:3, 2:3, 4:4, 2:5, 1:5,
	       2:6, 2:7, 3:7, 3:8, 3:9, 3:10, 2:10,
	       4:7, 5:7, 5:8, 5:9, 5:10, 5:11, 6:10,
	       7:10, 7:9, 7:8, 7:7, 8:7,
	       4:2, 7:3, 3:11, 7:11
	      ] ).

start( 1:5 ).

stop( 8:7 ).

step( 8:7, WL, PL ) :-
   init_maze,
   draw_maze( PL ),
   write("You found the Exit point !"),
   terminate_system.
step( I:J, WL, PL ) :-
   is_branch( I:J, WL ),
   start_south_process( I:J, WL, PL ),
   start_north_process( I:J, WL, PL ),
   start_east_process( I:J, WL, PL ),
   start_west_process( I:J, WL, PL ).
step( I:J, WL, PL ) :-
   step2( I:J, WL, PL ).
step( _, _, _ ).

step2( I:J, WL, PL ) :-
   draw_maze( PL ),
   K is I + 1,
   del_pos( K:J, WL, WL2 ),
   add_pos( K:J, PL, PL2 ),
   step( K:J, WL2, PL2 ).
step2( I:J, WL, PL ) :-
   draw_maze( PL ),
   K is I - 1,
   del_pos( K:J, WL, WL2 ),
   add_pos( K:J, PL, PL2 ),
   step( K:J, WL2, PL2 ).
step2( I:J, WL, PL ) :-
   draw_maze( PL ),
   K is J + 1,
   del_pos( I:K, WL, WL2 ),
   add_pos( I:K, PL, PL2 ),
   step( I:K, WL2, PL2 ).
step2( I:J, WL, PL ) :-
   draw_maze( PL ),
   K is J - 1,
   del_pos( I:K, WL, WL2 ),
   add_pos( I:K, PL, PL2 ),
   step( I:K, WL2, PL2 ).

step3( I:J, WL, PL ) :-
   init_maze,
   draw_maze( PL ),
   K is I + 1,
   del_pos( K:J, WL, WL2 ),
   add_pos( K:J, PL, PL2 ),
   step( K:J, WL2, PL2 ).

step4( I:J, WL, PL ) :-
   init_maze,
   draw_maze( PL ),
   K is I - 1,
   del_pos( K:J, WL, WL2 ),
   add_pos( K:J, PL, PL2 ),
   step( K:J, WL2, PL2 ).

step5( I:J, WL, PL ) :-
   init_maze,
   draw_maze( PL ),
   K is J + 1,
   del_pos( I:K, WL, WL2 ),
   add_pos( I:K, PL, PL2 ),
   step( I:K, WL2, PL2 ).

step6( I:J, WL, PL ) :-
   init_maze,
   draw_maze( PL ),
   K is J - 1,
   del_pos( I:K, WL, WL2 ),
   add_pos( I:K, PL, PL2 ),
   step( I:K, WL2, PL2 ).

del_pos( H, [H|T], T ).
del_pos( E, [H|T], [H|T2] ) :-
   del_pos( E, T, T2 ).

add_pos( H, T, [H|T] ).

member( H, [H|T] ).
member( E, [H|T] ) :-
   member( E, T ).

is_branch( I:J, WL ) :-
   delete_partition( branch ),
   set_value( counter, 0 ),
   branch_count( I:J, WL ),
   get_value( counter, C ),
   C > 1 .

branch_count( I:J, WL ) :-
   K is I + 1,
   member( K:J, WL ),
   add_clause( branch(s) ),
   get_value( counter, C ),
   CC is C + 1 ,
   set_value( counter, CC ),
   fail.
branch_count( I:J, WL ) :-
   K is I - 1,
   member( K:J, WL ),
   add_clause( branch(n) ),
   get_value( counter, C ),
   CC is C + 1 ,
   set_value( counter, CC ),
   fail.
branch_count( I:J, WL ) :-
   K is J + 1,
   member( I:K, WL ),
   add_clause( branch(e) ),
   get_value( counter, C ),
   CC is C + 1 ,
   set_value( counter, CC ),
   fail.
branch_count( I:J, WL ) :-
   K is J - 1,
   member( I:K, WL ),
   add_clause( branch(w) ),
   get_value( counter, C ),
   CC is C + 1 ,
   set_value( counter, CC ).
branch_count( _, _ ).

start_south_process( I:J, WL, PL ):-
   is_south_branch, !,
   new( step3( I:J, WL, PL ), _, _, _, 1 ).
start_south_process( _, _, _ ).

start_north_process( I:J, WL, PL ):-
   is_north_branch, !,
   new( step4( I:J, WL, PL ), _, _, _, 2 ).
start_north_process( _, _, _ ).

start_east_process( I:J, WL, PL ):-
   is_east_branch, !,
   new( step5( I:J, WL, PL ), _, _, _, 3 ).
start_east_process( _, _, _ ).

start_west_process( I:J, WL, PL ):-
   is_west_branch, !,
   new( step6( I:J, WL, PL ), _, _, _, 4 ).
start_west_process( _, _, _ ).

is_south_branch :-
   find_clause( branch(s) ).

is_north_branch :-
   find_clause( branch(n) ).

is_east_branch :-
   find_clause( branch(e) ).

is_west_branch :-
   find_clause( branch(w) ).

%-----------------------------------------------%
%	graphics
%-----------------------------------------------%

init_maze:-
   own_processor(OP),
   wind_id2( OP, Wind ),
   get_way_list( WL),
   get_way_draw(Wind,WL),
   wind_id( OP, Wind2 ),
   start( S ), write_start(Wind2, S ),
   stop( SP ), write_stop(Wind2, SP ).

draw(Tr):- get_way_list(WL),
	    transform(Tr,Pos1,Pos2),
	    wind_id(Tr,Wind2),
	    wind_id2(Tr,Wind1),
	    table(Wind1,Wind2,WL,Pos1,Pos2),
	    start( S ), write_start(Wind2, S ),
	    stop( SP ), write_stop(Wind2, SP ).

transform(1,2,2).
transform(2,2,42).
transform(3,15,2).
transform(4,15,42).

table(Wind1,Wind2,WL,I,J):-
		 create_window(I,J,8,24,7,Wind1),
		 create_window(I,J,8,24,124,Wind2),
		 open_window(Wind1),
		 get_way_draw(Wind1,WL).

write_start(Wind, X:Y ):-
	       Y1 is 2 * Y - 2,X1 is X - 1,
	       write_window(Wind,X1,Y1,"IN").

write_stop(Wind, X:Y ):-
	    Y1 is 2*Y - 2, X1 is X - 1,
	    write_window(Wind,X1,Y1,"EX").

get_way_draw(Wind,[]):- !.
get_way_draw(Wind,[X1:Y1|L]):-
		Y2 is 2*Y1 - 2,X2 is X1 - 1,
		write_window(Wind,X2,Y2,""),
		get_way_draw(Wind,L).

draw_maze( PL ) :-
   own_processor( OP ),
   wind_id( OP, Wind ),
   display_list( Wind, PL ).

wind_id( 1, w1 ).
wind_id( 2, w2 ).
wind_id( 3, w3 ).
wind_id( 4, w4 ).

wind_id2( 1, ww1 ).
wind_id2( 2, ww2 ).
wind_id2( 3, ww3 ).
wind_id2( 4, ww4 ).

display_list(Wind, [] ):- !.
display_list(Wind, [8:7|T] ) :- !, display_list(Wind, T ).
display_list(Wind, [X1:Y1|T] ) :-
   Y2 is 2*Y1 - 2, X2 is X1 - 1,
   write_window(Wind,X2,Y2,""),
   display_list( Wind, T ).

