% QUEENS.PRO
%---------------------------------------%
%  8-queens problem			%
%  written by				%
%     Peter Kacsuk and Agnes Zubora	%
%  based on solution of Ivan Bratko	%
%   (Modified by P.Z.)			%
%---------------------------------------%

start:-
     max_processor(MP),
     write_inside(["How many processors would you like to use ? (1 <= n <= ",
     		MP,")"]),
     nl,		
     write("  n = "),
     read_token(N),
     is_int(N),
     N <= MP, N > 0,
     !,
     problem(N).
start:-
     write("Sorry, parameter error. Please, try again. "), nl, nl,
     start.

problem(N):-
     open_window("system dialog window"),
     new(master(N,8),master).

%---------------------------------------%
%	master process			%
%---------------------------------------%

master(N,M):-
     cpu_time(X),
     solve(N,M),
     collect_end_signals(M),
     cpu_time(Y),
     T is Y - X,
     display_time(T).

collect_end_signals(0).
collect_end_signals(N):-
     wait_for(end),
     N1 is N - 1,
     collect_end_signals(N1).


solve(N,M):- solve1(N,M,0).

solve1(N,M,M):- !.
solve1(N,M,P):-
	Q is P + 1,
	TR is (Q mod N) + 1,
	new(queen_solve(Q,M),worker(Q),_,_,TR),
	solve1(N,M,Q).

display_time(T):-
   create_window(11,32,3,15,576,wt),
   open_window(wt),
   write_window(wt,0,7,"time"),
   write_to_string(ST,T),
   write_window(wt,1,1,ST).


%---------------------------------------%
%	worker process			%
%---------------------------------------%

queen_solve(Q,M):-
    one_solution(Q,M,Res),
    display_result(Res,Q,M),
    send(end,[master]).

one_solution(Q,M,Res):-
    solution(1,[Q],M,Res),	     %There is a solution
    !.
one_solution(_,_,[]).		     %There is no solution


range(I,L,H) :- is_var(I), !, L <= H, rg1(I,L,H).
range(I,L,H) :- L <= I, I <= H.

rg1(L,L,H).
rg1(I,L,H) :- L1 is L + 1, L1 <= H, rg1(I,L1,H).

solution(M,Res,M,Res) :- !.
solution(Depth,Part,M,Res) :-
	range(New,1,M),
	safe(Part,New,1),
	D1 is Depth + 1,
	solution(D1,[New|Part],M,Res).

safe([],_,_) :- !.
safe([H|T],New,Dist):-
	noattack(H,New,Dist),
	D1 is Dist + 1,
	safe(T,New,D1).

noattack(Q,Q,_) :- !, fail.
noattack(Q,P,D) :- P is Q + D, !, fail.
noattack(Q,P,D) :- P =\= Q - D .


%---------------------------------------%
%	graphics predicates		%
%---------------------------------------%

display_result(Res,Q,M):-
	table(Wind1,Wind2,Q,M),
	sol_draw_window(Wind1,Wind2,Res,1).

table(Wind1,Wind2,P,M):-
	calc_origin(P,Row0,Col0),
	create_window(Row0,Col0,10,18,263,Wind1),
	create_window(Row0,Col0,10,18,112,Wind2),
	Patt = "                          ",
	L is 2 * M,
	substring(Patt,1,L,Odd),
	substring(Patt,3,L,Even),
	open_window(Wind1),
	rows(Wind1,M,0,Odd,Even).

calc_origin(P,Row,Col):-
	P < 5,
	!,
	Row = 1,
	Col is 20 * (P - 1) + 1 .
calc_origin(P,Row,Col):-
	Row = 14,
	Col is 20 * (P - 5) + 1 .

rows(_,M,M,_,_):- !.		% provisionally for 8 by 8 board only
rows(Window,M,I,Odd,Even):-
	I1 is I + 1,
	select_mod2(Odd,Even,I1,Line),
	write_window(Window,I1,1,Line),
	rows(Window,M,I1,Odd,Even).


sol_draw_window(_,_,[],_):- !.
sol_draw_window(Wind1,Wind2,[Col|List],Row):-
	Mod is Col+Row,
	C2 is 2*Col - 1,
	select_mod2(Wind1,Wind2,Mod,W),
	write_window(W,Row,C2,""),
	R2 is Row + 1,
	sol_draw_window(Wind1,Wind2,List,R2).

select_mod2(A,_,N,A) :- 1 is N mod 2, !.
select_mod2(_,B,_,B).

